@@ -22,14 +22,11 @@
 #include "screen_find.hxx"
 #include "FileListPage.hxx"
 #include "Command.hxx"
-#include "screen.hxx"
 #include "i18n.h"
 #include "charset.hxx"
 #include "mpdclient.hxx"
 #include "util/StringUTF8.hxx"
 
-#include <glib.h>
-
 #include <algorithm>
 
 #include <assert.h>
@@ -46,7 +43,7 @@ CompareUTF8(const std::string &a, const std::string &b)
 
 const char *
 AlbumListPage::GetListItemText(char *buffer, size_t size,
-			       unsigned idx) const
+			       unsigned idx) const noexcept
 {
 	if (idx == 0)
 		return "..";
@@ -57,10 +54,7 @@ AlbumListPage::GetListItemText(char *buffer, size_t size,
 
 	assert(idx < album_list.size());
 
-	const char *str_utf8 = album_list[idx].c_str();
-
-	g_strlcpy(buffer, Utf8ToLocale(str_utf8).c_str(), size);
-	return buffer;
+	return utf8_to_locale(album_list[idx].c_str(), buffer, size);
 }
 
 static void
@@ -115,30 +109,26 @@ AlbumListPage::Reload(struct mpdclient &c)
 void
 AlbumListPage::PaintListItem(WINDOW *w, unsigned i,
 			     gcc_unused unsigned y, unsigned width,
-			     bool selected) const
+			     bool selected) const noexcept
 {
-	const char *p;
-	char *q = nullptr;
-
 	if (i == 0)
-		p = "..";
+		screen_browser_paint_directory(w, width, selected, "..");
 	else if (i == album_list.size() + 1)
-		p = _("All tracks");
+		screen_browser_paint_directory(w, width, selected,
+					       _("All tracks"));
 	else
-		p = q = utf8_to_locale(album_list[i - 1].c_str());
-
-	screen_browser_paint_directory(w, width, selected, p);
-	g_free(q);
+		screen_browser_paint_directory(w, width, selected,
+					       Utf8ToLocale(album_list[i - 1].c_str()).c_str());
 }
 
 void
-AlbumListPage::Paint() const
+AlbumListPage::Paint() const noexcept
 {
 	lw.Paint(*this);
 }
 
 const char *
-AlbumListPage::GetTitle(char *str, size_t size) const
+AlbumListPage::GetTitle(char *str, size_t size) const noexcept
 {
 	if (artist.empty())
 		return _("Albums");
@@ -149,7 +139,7 @@ AlbumListPage::GetTitle(char *str, size_t size) const
 }
 
 void
-AlbumListPage::Update(struct mpdclient &c, unsigned events)
+AlbumListPage::Update(struct mpdclient &c, unsigned events) noexcept
 {
 	if (events & MPD_IDLE_DATABASE) {
 		/* the db has changed -> update the list */
@@ -191,11 +181,9 @@ AlbumListPage::OnCommand(struct mpdclient &c, Command cmd)
 		const char *selected;
 
 	case Command::PLAY:
-		if (lw.selected == 0) {
+		if (lw.selected == 0 && parent != nullptr)
 			/* handle ".." */
-			screen.OnCommand(c, Command::GO_PARENT_DIRECTORY);
-			return true;
-		}
+			return parent->OnCommand(c, Command::GO_PARENT_DIRECTORY);
 
 		break;
 
@@ -223,12 +211,12 @@ AlbumListPage::OnCommand(struct mpdclient &c, Command cmd)
 	case Command::LIST_RFIND:
 	case Command::LIST_FIND_NEXT:
 	case Command::LIST_RFIND_NEXT:
-		screen_find(screen, &lw, cmd, *this);
+		screen_find(screen, lw, cmd, *this);
 		SetDirty();
 		return true;
 
 	case Command::LIST_JUMP:
-		screen_jump(screen, &lw, *this, *this);
+		screen_jump(screen, lw, *this, *this);
 		SetDirty();
 		return true;
 
