@@ -29,8 +29,6 @@
 
 #include <algorithm>
 
-#include <glib.h>
-
 #include <assert.h>
 #include <string.h>
 
@@ -45,14 +43,11 @@ CompareUTF8(const std::string &a, const std::string &b)
 
 const char *
 ArtistListPage::GetListItemText(char *buffer, size_t size,
-				unsigned idx) const
+				unsigned idx) const noexcept
 {
 	assert(idx < artist_list.size());
 
-	const char *str_utf8 = artist_list[idx].c_str();
-
-	g_strlcpy(buffer, Utf8ToLocale(str_utf8).c_str(), size);
-	return buffer;
+	return utf8_to_locale(artist_list[idx].c_str(), buffer, size);
 }
 
 static void
@@ -96,26 +91,26 @@ ArtistListPage::Reload(struct mpdclient &c)
 void
 ArtistListPage::PaintListItem(WINDOW *w, unsigned i,
 			      gcc_unused unsigned y, unsigned width,
-			      bool selected) const
+			      bool selected) const noexcept
 {
 	screen_browser_paint_directory(w, width, selected,
 				       Utf8ToLocale(artist_list[i].c_str()).c_str());
 }
 
 void
-ArtistListPage::Paint() const
+ArtistListPage::Paint() const noexcept
 {
 	lw.Paint(*this);
 }
 
 const char *
-ArtistListPage::GetTitle(char *, size_t) const
+ArtistListPage::GetTitle(char *, size_t) const noexcept
 {
 	return _("All artists");
 }
 
 void
-ArtistListPage::Update(struct mpdclient &c, unsigned events)
+ArtistListPage::Update(struct mpdclient &c, unsigned events) noexcept
 {
 	if (events & MPD_IDLE_DATABASE) {
 		/* the db has changed -> update the list */
@@ -183,12 +178,12 @@ ArtistListPage::OnCommand(struct mpdclient &c, Command cmd)
 	case Command::LIST_RFIND:
 	case Command::LIST_FIND_NEXT:
 	case Command::LIST_RFIND_NEXT:
-		screen_find(screen, &lw, cmd, *this);
+		screen_find(screen, lw, cmd, *this);
 		SetDirty();
 		return true;
 
 	case Command::LIST_JUMP:
-		screen_jump(screen, &lw, *this, *this);
+		screen_jump(screen, lw, *this, *this);
 		SetDirty();
 		return true;
 
