@@ -29,7 +29,7 @@
 #include "config.h"
 #include "i18n.h"
 #include "util/Macros.hxx"
-#include "util/StringUTF8.hxx"
+#include "util/LocaleString.hxx"
 
 #include <assert.h>
 
@@ -231,23 +231,23 @@ public:
 	/* virtual methods from class ListRenderer */
 	void PaintListItem(WINDOW *w, unsigned i,
 			   unsigned y, unsigned width,
-			   bool selected) const override;
+			   bool selected) const noexcept override;
 
 	/* virtual methods from class ListText */
 	const char *GetListItemText(char *buffer, size_t size,
-				    unsigned i) const override;
+				    unsigned i) const noexcept override;
 
 	/* virtual methods from class Page */
-	void Paint() const override;
+	void Paint() const noexcept override;
 	bool OnCommand(struct mpdclient &c, Command cmd) override;
 
-	const char *GetTitle(char *, size_t) const override {
+	const char *GetTitle(char *, size_t) const noexcept override {
 		return _("Help");
 	}
 };
 
 const char *
-HelpPage::GetListItemText(char *, size_t, unsigned i) const
+HelpPage::GetListItemText(char *, size_t, unsigned i) const noexcept
 {
 	const auto *row = &help_text[i];
 
@@ -271,7 +271,7 @@ help_init(ScreenManager &screen, WINDOW *w, Size size)
 void
 HelpPage::PaintListItem(WINDOW *w, unsigned i,
 			unsigned y, unsigned width,
-			gcc_unused bool selected) const
+			gcc_unused bool selected) const noexcept
 {
 	const auto *row = &help_text[i];
 
@@ -287,12 +287,12 @@ HelpPage::PaintListItem(WINDOW *w, unsigned i,
 		else if (row->highlight == 2)
 			mvwhline(w, y, 3, ACS_HLINE, width - 6);
 	} else {
-		const char *key =
-			GetGlobalKeyBindings().GetKeyNames(row->command, true);
+		const auto key =
+			GetGlobalKeyBindings().GetKeyNames(row->command);
 
-		if (utf8_width(key) < 20)
-			wmove(w, y, 20 - utf8_width(key));
-		waddstr(w, key);
+		if (StringWidthMB(key.c_str()) < 20)
+			wmove(w, y, 20 - StringWidthMB(key.c_str()));
+		waddstr(w, key.c_str());
 		mvwaddch(w, y, 21, ':');
 		mvwaddstr(w, y, 23,
 			  row->text != nullptr
@@ -302,7 +302,7 @@ HelpPage::PaintListItem(WINDOW *w, unsigned i,
 }
 
 void
-HelpPage::Paint() const
+HelpPage::Paint() const noexcept
 {
 	lw.Paint(*this);
 }
@@ -314,7 +314,7 @@ HelpPage::OnCommand(struct mpdclient &c, Command cmd)
 		return true;
 
 	lw.SetCursor(lw.start);
-	if (screen_find(screen, &lw, cmd, *this)) {
+	if (screen_find(screen, lw, cmd, *this)) {
 		/* center the row */
 		lw.Center(lw.selected);
 		SetDirty();
