@@ -28,26 +28,22 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
-#include <glib.h>
 
 #define ERROR_UNKNOWN_OPTION    0x01
 #define ERROR_BAD_ARGUMENT      0x02
 #define ERROR_GOT_ARGUMENT      0x03
 #define ERROR_MISSING_ARGUMENT  0x04
 
-typedef struct {
+struct OptionDefinition {
 	int shortopt;
 	const char *longopt;
 	const char *argument;
 	const char *descrition;
-} arg_opt_t;
-
-
-typedef void (*option_callback_fn_t)(int c, const char *arg);
+};
 
 Options options;
 
-static const arg_opt_t option_table[] = {
+static constexpr OptionDefinition option_table[] = {
 	{ '?', "help", nullptr, "Show this help message" },
 	{ 'V', "version", nullptr, "Display version information" },
 	{ 'c', "colors", nullptr, "Enable colors" },
@@ -67,19 +63,23 @@ static const arg_opt_t option_table[] = {
 #endif
 };
 
-static const unsigned option_table_size = sizeof(option_table) / sizeof(option_table[0]);
-
-static const arg_opt_t *
-lookup_option(int s, char *l)
+gcc_pure
+static const OptionDefinition *
+FindOption(int s) noexcept
 {
-	unsigned i;
+	for (const auto &i : option_table)
+		if (s == i.shortopt)
+			return &i;
 
-	for (i = 0; i < option_table_size; ++i) {
-		if (l && strcmp(l, option_table[i].longopt) == 0)
-			return &option_table[i];
-		if (s && s == option_table[i].shortopt)
-			return &option_table[i];
-	}
+	return nullptr;
+}
+
+static const OptionDefinition *
+FindOption(const char *l) noexcept
+{
+	for (const auto &i : option_table)
+		if (strcmp(l, i.longopt) == 0)
+			return &i;
 
 	return nullptr;
 }
@@ -114,20 +114,21 @@ display_help()
 {
 	printf("Usage: %s [OPTION]...\n", PACKAGE);
 
-	for (unsigned i = 0; i < option_table_size; ++i) {
+	for (const auto &i : option_table) {
 		char tmp[32];
 
-		if (option_table[i].argument)
+		if (i.argument)
 			snprintf(tmp, sizeof(tmp), "%s=%s",
-				 option_table[i].longopt,
-				 option_table[i].argument);
+				 i.longopt,
+				 i.argument);
 		else
-			g_strlcpy(tmp, option_table[i].longopt, 64);
+			snprintf(tmp, sizeof(tmp), "%s",
+				 i.longopt);
 
 		printf("  -%c, --%-20s %s\n",
-		       option_table[i].shortopt,
+		       i.shortopt,
 		       tmp,
-		       option_table[i].descrition);
+		       i.descrition);
 	}
 }
 
@@ -156,6 +157,9 @@ handle_option(int c, const char *arg)
 #ifdef ENABLE_LOCALE
 		     " locale"
 #endif
+#ifdef HAVE_ICONV
+		     " iconv"
+#endif
 #ifdef ENABLE_NLS
 		     " nls"
 #endif
@@ -270,34 +274,30 @@ handle_option(int c, const char *arg)
 void
 options_parse(int argc, const char *argv[])
 {
-	const arg_opt_t *opt = nullptr;
-	option_callback_fn_t option_cb = handle_option;
+	const OptionDefinition *opt = nullptr;
 
 	for (int i = 1; i < argc; i++) {
 		const char *arg = argv[i];
 		size_t len = strlen(arg);
 
 		/* check for a long option */
-		if (g_str_has_prefix(arg, "--")) {
-			char *name, *value;
-
+		if (strncmp(arg, "--", 2) == 0) {
 			/* make sure we got an argument for the previous option */
 			if( opt && opt->argument )
 				option_error(ERROR_MISSING_ARGUMENT, opt->longopt, opt->argument);
 
 			/* retrieve a option argument */
-			if ((value=g_strrstr(arg+2, "="))) {
-				*value = '\0';
-				name = g_strdup(arg);
-				*value = '=';
+			std::string name;
+			const char *value = strrchr(arg + 2, '=');
+			if (value != nullptr) {
+				name.assign(arg, value);
 				value++;
 			} else
-				name = g_strdup(arg);
+				name = arg;
 
 			/* check if the option exists */
-			if( (opt=lookup_option(0, name+2)) == nullptr )
-				option_error(ERROR_UNKNOWN_OPTION, name, nullptr);
-			g_free(name);
+			if ((opt = FindOption(name.c_str() + 2)) == nullptr)
+				option_error(ERROR_UNKNOWN_OPTION, name.c_str(), nullptr);
 
 			/* abort if we got an argument to the option and don't want one */
 			if( value && opt->argument==nullptr )
@@ -305,12 +305,12 @@ options_parse(int argc, const char *argv[])
 
 			/* execute option callback */
 			if (value || opt->argument==nullptr) {
-				option_cb (opt->shortopt, value);
+				handle_option(opt->shortopt, value);
 				opt = nullptr;
 			}
 		}
 		/* check for short options */
-		else if (len>=2 && g_str_has_prefix(arg, "-")) {
+		else if (len >= 2 && *arg == '-') {
 			size_t j;
 
 			for(j=1; j<len; j++) {
@@ -320,19 +320,19 @@ options_parse(int argc, const char *argv[])
 						     opt->longopt, opt->argument);
 
 				/* check if the option exists */
-				if ((opt=lookup_option(arg[j], nullptr)) == nullptr)
+				if ((opt = FindOption(arg[j])) == nullptr)
 					option_error(ERROR_UNKNOWN_OPTION, arg, nullptr);
 
 				/* if no option argument is needed execute callback */
 				if (opt->argument == nullptr) {
-					option_cb (opt->shortopt, nullptr);
+					handle_option(opt->shortopt, nullptr);
 					opt = nullptr;
 				}
 			}
 		} else {
 			/* is this a option argument? */
 			if (opt && opt->argument) {
-				option_cb (opt->shortopt, arg);
+				handle_option (opt->shortopt, arg);
 				opt = nullptr;
 			} else
 				option_error(ERROR_BAD_ARGUMENT, arg, nullptr);
@@ -340,7 +340,7 @@ options_parse(int argc, const char *argv[])
 	}
 
 	if (opt && opt->argument == nullptr)
-		option_cb (opt->shortopt, nullptr);
+		handle_option(opt->shortopt, nullptr);
 	else if (opt && opt->argument)
 		option_error(ERROR_MISSING_ARGUMENT, opt->longopt, opt->argument);
 
