@@ -35,12 +35,16 @@
 #include "util/StringStrip.hxx"
 
 #include <assert.h>
+#include <sys/stat.h>
+#include <ctype.h>
 #include <stdio.h>
 #include <errno.h>
 #include <stdlib.h>
 #include <string.h>
+
+#ifdef _WIN32
 #include <glib.h>
-#include <glib/gstdio.h>
+#endif
 
 #define MAX_LINE_LENGTH 1024
 #define COMMENT_TOKEN '#'
@@ -163,39 +167,30 @@ static bool
 parse_key_definition(char *str)
 {
 	/* get the command name */
-	const size_t len = strlen(str);
-	size_t i = 0;
-	int j = 0;
-	char buf[MAX_LINE_LENGTH];
-	memset(buf, 0, MAX_LINE_LENGTH);
-	while (i < len && str[i] != '=' && !IsWhitespaceFast(str[i]))
-		buf[j++] = str[i++];
-
-	const auto cmd = get_key_command_from_name(buf);
-	if(cmd == Command::NONE) {
-		/* the hotkey configuration contains an unknown
-		   command */
-		print_error(_("Unknown command"), buf);
-		return false;
-	}
-
-	/* skip whitespace */
-	while (i < len && (str[i] == '=' || IsWhitespaceFast(str[i])))
-		i++;
-
-	/* get the value part */
-	memset(buf, 0, MAX_LINE_LENGTH);
-	g_strlcpy(buf, str+i, MAX_LINE_LENGTH);
-	if (*buf == 0) {
+	char *eq = strchr(str, '=');
+	if (eq == nullptr) {
 		/* the hotkey configuration line is incomplete */
 		print_error(_("Incomplete hotkey configuration"), str);
 		return false;
 	}
 
+	char *command_name = str;
+	str = StripLeft(eq + 1);
+
+	*eq = '\0';
+	StripRight(command_name);
+	const auto cmd = get_key_command_from_name(command_name);
+	if(cmd == Command::NONE) {
+		/* the hotkey configuration contains an unknown
+		   command */
+		print_error(_("Unknown command"), command_name);
+		return false;
+	}
+
 	/* parse key values */
-	i = 0;
+	size_t i = 0;
 	int key = 0;
-	char *p = buf;
+	char *p = str;
 
 	std::array<int, MAX_COMMAND_KEYS> keys{0};
 	while (i < MAX_COMMAND_KEYS && *p != 0 &&
@@ -243,8 +238,7 @@ separate_value(char *p)
 
 	*value++ = 0;
 
-	g_strchomp(p);
-	return StripLeft(value);
+	return Strip(value);
 }
 
 static bool
@@ -273,7 +267,7 @@ after_comma(char *p)
 	} else
 		comma = p + strlen(p);
 
-	g_strchomp(p);
+	StripRight(p);
 	return comma;
 }
 
@@ -334,32 +328,51 @@ GetStringValue(const char *s)
 	return {s, length};
 }
 
+static constexpr bool
+IsListSeparator(char ch) noexcept
+{
+	return IsWhitespaceFast(ch) || ch == ',';
+}
+
+static char *
+NextItem(char *&src) noexcept
+{
+	while (*src && IsListSeparator(*src))
+		++src;
+
+	if (!*src)
+		return nullptr;
+
+	char *value = src;
+
+	while (!IsListSeparator(*src))
+		++src;
+
+	if (*src)
+		*src++ = 0;
+
+	return value;
+}
+
 static std::vector<std::string>
 check_screen_list(char *value)
 {
-	char **tmp = g_strsplit_set(value, " \t,", 100);
-	AtScopeExit(tmp) { g_strfreev(tmp); };
-
 	std::vector<std::string> screen;
-	int i = 0;
 
-	while( tmp && tmp[i] ) {
-		char *name = g_ascii_strdown(tmp[i], -1);
-		AtScopeExit(name) { g_free(name); };
-		if (*name != '\0') {
-			if (screen_lookup_name(name) == nullptr) {
-				/* an unknown screen name was specified in the
-				   configuration file */
-				print_error(_("Unknown screen name"), name);
-			} else {
-				screen.emplace_back(name);
-			}
+	while (char *name = NextItem(value)) {
+		std::transform(name, name + strlen(name), name, tolower);
+
+		if (screen_lookup_name(name) == nullptr) {
+			/* an unknown screen name was specified in the
+			   configuration file */
+			print_error(_("Unknown screen name"), name);
+		} else {
+			screen.emplace_back(name);
 		}
-		i++;
 	}
 
 	if (screen.empty())
-		return DEFAULT_SCREEN_LIST;
+		screen = DEFAULT_SCREEN_LIST;
 
 	return screen;
 }
@@ -506,7 +519,7 @@ parse_line(char *line)
 	else if (!strcasecmp(CONF_SEARCH_MODE, name))
 		options.search_mode = get_search_mode(value);
 	else if (!strcasecmp(CONF_HIDE_CURSOR, name))
-		options.hide_cursor = atoi(value);
+		options.hide_cursor = std::chrono::seconds(atoi(value));
 	else if (!strcasecmp(CONF_SEEK_TIME, name))
 		options.seek_time = atoi(value);
 	else if (!strcasecmp(CONF_SCREEN_LIST, name)) {
@@ -522,7 +535,7 @@ parse_line(char *line)
 				     * 1000 /* seconds -> milliseconds */;
 	else if (!strcasecmp(CONF_LYRICS_TIMEOUT, name))
 #ifdef ENABLE_LYRICS_SCREEN
-		options.lyrics_timeout = atoi(GetStringValue(value).c_str());
+		options.lyrics_timeout = std::chrono::seconds(atoi(GetStringValue(value).c_str()));
 #else
 	{}
 #endif
@@ -585,7 +598,7 @@ parse_line(char *line)
 	return true;
 }
 
-static int
+static bool
 read_rc_file(const char *filename)
 {
 	assert(filename != nullptr);
@@ -593,21 +606,76 @@ read_rc_file(const char *filename)
 	FILE *file = fopen(filename, "r");
 	if (file == nullptr) {
 		perror(filename);
-		return -1;
+		return false;
 	}
 
 	char line[MAX_LINE_LENGTH];
 	while (fgets(line, sizeof(line), file) != nullptr) {
 		char *p = StripLeft(line);
 
-		if (*p != 0 && *p != COMMENT_TOKEN)
-			parse_line(g_strchomp(p));
+		if (*p != 0 && *p != COMMENT_TOKEN) {
+			StripRight(p);
+			parse_line(p);
+		}
 	}
 
 	fclose(file);
-	return 0;
+	return true;
 }
 
+gcc_pure
+static bool
+IsFile(const char *path) noexcept
+{
+	struct stat st;
+	return stat(path, &st) == 0 && S_ISREG(st.st_mode);
+}
+
+gcc_pure
+static bool
+IsDirectory(const char *path) noexcept
+{
+	struct stat st;
+	return stat(path, &st) == 0 && S_ISDIR(st.st_mode);
+}
+
+#ifndef _WIN32
+
+gcc_const
+static const char *
+GetHomeDirectory() noexcept
+{
+	return getenv("HOME");
+}
+
+gcc_const
+static std::string
+GetHomeConfigDirectory() noexcept
+{
+	const char *config_home = getenv("XDG_CONFIG_HOME");
+	if (config_home != nullptr && *config_home != 0)
+		return config_home;
+
+	const char *home = GetHomeDirectory();
+	if (home != nullptr)
+		return BuildPath(home, ".config");
+
+	return {};
+}
+
+gcc_pure
+static std::string
+GetHomeConfigDirectory(const char *package) noexcept
+{
+	const auto dir = GetHomeConfigDirectory();
+	if (dir.empty())
+		return {};
+
+	return BuildPath(dir, package);
+}
+
+#endif
+
 /**
  * Find or create the directory for writing configuration files.
  *
@@ -617,10 +685,12 @@ read_rc_file(const char *filename)
 static std::string
 MakeUserConfigPath(const char *filename)
 {
-	const auto directory = BuildPath(g_get_user_config_dir(), PACKAGE);
+	const auto directory = GetHomeConfigDirectory(PACKAGE);
+	if (directory.empty())
+		return {};
 
-	return g_file_test(directory.c_str(), G_FILE_TEST_IS_DIR) ||
-		g_mkdir(directory.c_str(), 0755) == 0
+	return IsDirectory(directory.c_str()) ||
+		mkdir(directory.c_str(), 0755) == 0
 		? BuildPath(directory, filename)
 		: std::string();
 }
@@ -636,7 +706,11 @@ MakeKeysPath()
 std::string
 GetHomeConfigPath()
 {
-	return BuildPath(g_get_home_dir(), "." PACKAGE, CONFIG_FILENAME);
+	const char *home = GetHomeDirectory();
+	if (home == nullptr)
+		return {};
+
+	return BuildPath(home, "." PACKAGE, CONFIG_FILENAME);
 }
 
 #endif
@@ -644,7 +718,11 @@ GetHomeConfigPath()
 std::string
 GetUserConfigPath()
 {
-	return BuildPath(g_get_user_config_dir(), PACKAGE, CONFIG_FILENAME);
+	const auto dir = GetHomeConfigDirectory();
+	if (dir.empty())
+		return {};
+
+	return BuildPath(dir, PACKAGE, CONFIG_FILENAME);
 }
 
 std::string
@@ -656,7 +734,7 @@ GetSystemConfigPath()
 	for (system_data_dirs = g_get_system_config_dirs (); *system_data_dirs != nullptr; system_data_dirs++)
 	{
 		auto path = BuildPath(*system_data_dirs, PACKAGE, CONFIG_FILENAME);
-		if (g_file_test(path.c_str(), G_FILE_TEST_EXISTS))
+		if (IsFile(path.c_str()))
 			return path;
 	}
 	return {};
@@ -671,7 +749,11 @@ gcc_pure
 static std::string
 GetHomeKeysPath()
 {
-	return BuildPath(g_get_home_dir(), "." PACKAGE, KEYS_FILENAME);
+	const char *home = GetHomeDirectory();
+	if (home == nullptr)
+		return {};
+
+	return BuildPath(home, "." PACKAGE, KEYS_FILENAME);
 }
 
 #endif
@@ -680,7 +762,11 @@ gcc_pure
 static std::string
 GetUserKeysPath()
 {
-	return BuildPath(g_get_user_config_dir(), PACKAGE, KEYS_FILENAME);
+	const auto dir = GetHomeConfigDirectory();
+	if (dir.empty())
+		return {};
+
+	return BuildPath(dir, PACKAGE, KEYS_FILENAME);
 }
 
 gcc_pure
@@ -693,7 +779,7 @@ GetSystemKeysPath()
 	for (system_data_dirs = g_get_system_config_dirs (); *system_data_dirs != nullptr; system_data_dirs++)
 	{
 		auto path = BuildPath(*system_data_dirs, PACKAGE, KEYS_FILENAME);
-		if (g_file_test(pathname.c_str(), G_FILE_TEST_EXISTS))
+		if (IsFile(pathname.c_str()))
 			return path;
 	}
 	return {}
@@ -711,19 +797,19 @@ find_config_file()
 
 	/* check for user configuration ~/.config/ncmpc/config */
 	auto filename = GetUserConfigPath();
-	if (g_file_test(filename.c_str(), G_FILE_TEST_IS_REGULAR))
+	if (!filename.empty() && IsFile(filename.c_str()))
 		return filename;
 
 #ifndef _WIN32
 	/* check for user configuration ~/.ncmpc/config */
 	filename = GetHomeConfigPath();
-	if (g_file_test(filename.c_str(), G_FILE_TEST_IS_REGULAR))
+	if (!filename.empty() && IsFile(filename.c_str()))
 		return filename;
 #endif
 
 	/* check for  global configuration SYSCONFDIR/ncmpc/config */
 	filename = GetSystemConfigPath();
-	if (g_file_test(filename.c_str(), G_FILE_TEST_IS_REGULAR))
+	if (IsFile(filename.c_str()))
 		return filename;
 
 	return {};
@@ -738,19 +824,19 @@ find_keys_file()
 
 	/* check for user key bindings ~/.config/ncmpc/keys */
 	auto filename = GetUserKeysPath();
-	if (g_file_test(filename.c_str(), G_FILE_TEST_IS_REGULAR))
+	if (!filename.empty() && IsFile(filename.c_str()))
 		return filename;
 
 #ifndef _WIN32
 	/* check for  user key bindings ~/.ncmpc/keys */
 	filename = GetHomeKeysPath();
-	if (g_file_test(filename.c_str(), G_FILE_TEST_IS_REGULAR))
+	if (!filename.empty() && IsFile(filename.c_str()))
 		return filename;
 #endif
 
 	/* check for  global key bindings SYSCONFDIR/ncmpc/keys */
 	filename = GetSystemKeysPath();
-	if (g_file_test(filename.c_str(), G_FILE_TEST_IS_REGULAR))
+	if (IsFile(filename.c_str()))
 		return filename;
 
 	return {};
