@@ -42,6 +42,8 @@
 #include <string.h>
 
 class SongListPage final : public FileListPage {
+	Page *const parent;
+
 	std::string artist;
 
 	/**
@@ -52,9 +54,11 @@ class SongListPage final : public FileListPage {
 	std::string album;
 
 public:
-	SongListPage(ScreenManager &_screen, WINDOW *_w, Size size)
+	SongListPage(ScreenManager &_screen, Page *_parent,
+		     WINDOW *_w, Size size) noexcept
 		:FileListPage(_screen, _w, size,
-			      options.list_format.c_str()) {}
+			      options.list_format.c_str()),
+		 parent(_parent) {}
 
 	template<typename A>
 	void SetArtist(A &&_artist) {
@@ -79,13 +83,13 @@ public:
 	void LoadSongList(struct mpdclient &c);
 
 	/* virtual methods from class Page */
-	void Update(struct mpdclient &c, unsigned events) override;
+	void Update(struct mpdclient &c, unsigned events) noexcept override;
 	bool OnCommand(struct mpdclient &c, Command cmd) override;
-	const char *GetTitle(char *s, size_t size) const override;
+	const char *GetTitle(char *s, size_t size) const noexcept override;
 };
 
 void
-SongListPage::Update(struct mpdclient &c, unsigned events)
+SongListPage::Update(struct mpdclient &c, unsigned events) noexcept
 {
 	if (events & MPD_IDLE_DATABASE) {
 		LoadSongList(c);
@@ -102,8 +106,8 @@ public:
 			  Size size)
 		:ProxyPage(_w),
 		 artist_list_page(_screen, _w, size),
-		 album_list_page(_screen, _w, size),
-		 song_list_page(_screen, _w, size) {}
+		 album_list_page(_screen, this, _w, size),
+		 song_list_page(_screen, this, _w, size) {}
 
 private:
 	void OpenArtistList(struct mpdclient &c);
@@ -113,8 +117,8 @@ private:
 
 public:
 	/* virtual methods from class Page */
-	void OnOpen(struct mpdclient &c) override;
-	void Update(struct mpdclient &c, unsigned events) override;
+	void OnOpen(struct mpdclient &c) noexcept override;
+	void Update(struct mpdclient &c, unsigned events) noexcept override;
 	bool OnCommand(struct mpdclient &c, Command cmd) override;
 };
 
@@ -177,7 +181,7 @@ screen_artist_init(ScreenManager &_screen, WINDOW *w, Size size)
 }
 
 const char *
-SongListPage::GetTitle(char *str, size_t size) const
+SongListPage::GetTitle(char *str, size_t size) const noexcept
 {
 	const Utf8ToLocale artist_locale(artist.c_str());
 
@@ -203,11 +207,9 @@ SongListPage::OnCommand(struct mpdclient &c, Command cmd)
 {
 	switch(cmd) {
 	case Command::PLAY:
-		if (lw.selected == 0) {
+		if (lw.selected == 0 && parent != nullptr)
 			/* handle ".." */
-			screen.OnCommand(c, Command::GO_PARENT_DIRECTORY);
-			return true;
-		}
+			return parent->OnCommand(c, Command::GO_PARENT_DIRECTORY);
 
 		break;
 
@@ -224,7 +226,7 @@ SongListPage::OnCommand(struct mpdclient &c, Command cmd)
 }
 
 void
-ArtistBrowserPage::OnOpen(struct mpdclient &c)
+ArtistBrowserPage::OnOpen(struct mpdclient &c) noexcept
 {
 	ProxyPage::OnOpen(c);
 
@@ -233,7 +235,7 @@ ArtistBrowserPage::OnOpen(struct mpdclient &c)
 }
 
 void
-ArtistBrowserPage::Update(struct mpdclient &c, unsigned events)
+ArtistBrowserPage::Update(struct mpdclient &c, unsigned events) noexcept
 {
 	artist_list_page.AddPendingEvents(events);
 	album_list_page.AddPendingEvents(events);
